import RemoteApi_pb2
import remoteapi

"""
TestAntenna.py

Unit test for Remote API Antenna functionality
"""
client = remoteapi.DiscoveryClient().createClient()
try:
	if client is None:
		raise Exception("Failed to connect.")

	pi = client.pluginApi()
	piData = pi.data()
	piSimdis = pi.simdis()

	# Load data file with beams to attach antennas to
	f = piData.loadDataFile("$(SIMDIS_DIR)/demos/SIMDIS/Users/SADM/simdis_pres_20008_0001.asi")
	print("Requested load of data file.")
	if not f.isOk(45000):
		raise Exception("File load failed")


	# Begin testing simgple Set/Get methods ---------------
	# Ensure the scenario got loaded properly with a find-beam
	f = piData.findBeam('Beam 12000')
	if not f.isOk(1000) or f.value() == 0:
		raise Exception("Failed findBeam on Beam 12000")
	beam12000 = f.value()

	gainVal = 25.3
	f = piSimdis.setAntennaGain(beam12000, gainVal)
	if not f.isOk(1000):
		raise Exception("Failed setAntennaGain: " + f.responseDescription())

	f = piSimdis.getAntennaGain(beam12000)
	if not f.isOk(1000):
		raise Exception("Failed getAntennaGain: " + f.responseDescription())
	if f.value() != gainVal:
		raise Exception("Gain [" + str(f.value()) + "] does not match given [" + str(gainVal) + "]")

	powerVal = 9001
	f = piSimdis.setAntennaPower(beam12000, powerVal)
	if not f.isOk(1000):
		raise Exception("Failed setAntennaPower: " +  + f.responseDescription())

	f = piSimdis.getAntennaPower(beam12000)
	if not f.isOk(1000):
		raise Exception("Failed getAntennaPower: " + f.responseDescription())
	if f.value() != powerVal:
		raise Exception("Power [" + str(f.value()) + "] does not match given [" + str(powerVal) + "]")

	freqVal = 4242
	f = piSimdis.setAntennaFrequency(beam12000, freqVal)
	if not f.isOk(1000):
		raise Exception("Failed setAntennaFrequency: " + f.responseDescription())

	f = piSimdis.getAntennaFrequency(beam12000)
	if not f.isOk(1000):
		raise Exception("Failed getAntennaFrequency: " + f.responseDescription())
	if f.value() != freqVal:
		raise Exception("Frequency [" + str(f.value()) + "] does not match given [" + str(freqVal) + "]")

	polVal = 1
	f = piSimdis.setAntennaPolarity(beam12000,polVal)
	if not f.isOk(1000):
		raise Exception("Failed setAntennaPolarity: " + f.responseDescription())

	f = piSimdis.getAntennaPolarity(beam12000)
	if not f.isOk(1000):
		raise Exception("Failed getAntennaPolarity: " + f.responseDescription())
	if f.value() != polVal:
		raise Exception("Polarity [" + str(f.value()) + "] does not match given [" + str(polVal) + "]")


	# Begin testing advanced methods -------------------
	f = piData.findBeam('Beam 5001')
	if not f.isOk(1000) or f.value() == 0:
		raise Exception("Failed findBeam on Beam 5001")
	beam5001 = f.value()

	setName = 1 #1 = algo, others are different file patterns
	setAlgo = 2
	setPol = 2
	setFilename = "$(SIMDIS_DIR)/demos/SIMDIS/import/ant_sidelobes.aptf"
	setFilenameStart = setFilename.rfind("/")
	setGain = 17
	setPower = 12021
	setFreq = 11211
	f = piSimdis.setAntennaInfo(beam5001, setName, setAlgo, setPol, setFilename, setGain, setPower, setFreq)
	if not f.isOk(1500):
		raise Exception("Failed setAntennaInfo: " + f.responseDescription())


	f = piSimdis.getAntennaName(beam5001)
	if not f.isOk(1000):
		raise Exception("Failed getAntennaName: " + f.responseDescription())
	if f.value() != setName:
		raise Exception("AntennaName [" + str(f.value()) + "] does not match given [" + str(setName) + "]")

	f = piSimdis.getAntennaAlgorithm(beam5001)
	if not f.isOk(1000):
		raise Exception("Failed getAntennaAlgorithm: " + f.responseDescription())
	if f.value() != setAlgo:
		raise Exception("AntennaAlgorithm [" + str(f.value()) + "] does not match given [" + str(setAlgo) + "]")

	f = piSimdis.getAntennaFilename(beam5001)
	if not f.isOk(1000):
		raise Exception("Failed getFilename: " + f.responseDescription())
	if setName != 1:
		responseFilenameStart = f.value().rfind("/")
		if f.value[responseFilenameStart:] != setFilename[setFilenameStart:]:
			raise Exception("AntennaFilename [" + str(f.value()) + "] does not match given [" + str(setFilename) + "]")

	f = piSimdis.getAntennaPatternGain(beam5001, 0, 0)
	if not f.isOk(1000):
		raise Exception("Failed getPatternGain: " + f.responseDescription())
	if abs(f.value() - setGain) > 0.01:
		print("Warning: AntennaPatternGain not within threshold of setGain")

	f = piSimdis.clearAntennaInfo(beam5001)
	if not f.isOk(1000):
		raise Exception("Failed clearAntennaInfo: " + f.responseDescription())

	# Since clear*Info does not return anything, must manually check if the information has been cleared
	f = piSimdis.getAntennaName(beam5001)
	if not f.isOk(1000):
		raise Exception("Failed getAntennaName (after clear): " + f.responseDescription())
	if f.value() != 0:
		raise Exception("AntennaName after clear was not NONE")

	# All done, successful if we got here
	print("Test successful")
finally:
	client.close()