import math
import RemoteApi_pb2
import remoteapi

"""
TestBodyOffset.py

Unit test for Remote API Body Offset functions
"""

ReturnCodeOnly = lambda r: r.response.returnCode

client = remoteapi.DiscoveryClient().createClient()
if client is None:
	raise Exception("Failed to connect.")

pi = client.pluginApi()
piData = pi.data()
piSimdis = pi.simdis()

f = piData.deleteScenario()
if not f.isOk(1000):
	raise Exception("Failed deleteScenario")

# Load data file is unique because it can take a while to respond...
f = piData.loadDataFile("$(SIMDIS_DIR)/demos/SIMDIS/import/wake.asi")
print("Requested load of data file, waiting up to 50 seconds")
if not f.isOk(50000):
	raise Exception("Failed loadDataFile: " + f.responseDescription())

# Find both the beam and a platform
f = piData.findPlatform('PC-12')
if not f.isOk(1000) or f.value() == 0:
	raise Exception("Failed findPlatform on PC-12")
pc12Id = f.value()
f = piData.findBeam("Torp Beam")
if not f.isOk(1000) or f.value() == 0:
	raise Exception("Failed findBeam on Torp Beam")
torpBeamId = f.value()

# Get the original orientation
f = piData.getObjectOrientationOffset(pc12Id);
if not f.isOk(1000):
	raise Exception("Failed getObjectOrientationOffset 1: " + f.responseDescription())
originalOri = f.value()

# Set the orientation offset on the PC-12 to 135 degrees; should face north-ish
ROTATION_ANGLE = math.radians(135.0)
f = piData.setObjectOrientationOffset(pc12Id, [ROTATION_ANGLE, 0.0, 0.0])
if not f.isOk(1000):
	raise Exception("Failed setObjectOrientationOffset: " + f.responseDescription())

# Get the new orientation
f = piData.getObjectOrientationOffset(pc12Id);
if not f.isOk(1000):
	raise Exception("Failed getObjectOrientationOffset 2: " + f.responseDescription())
newOri = f.value()

# Define a helper for determining if two values are close enough
# http://stackoverflow.com/questions/5595425/what-is-the-best-way-to-compare-floats-for-almost-equality-in-python
def isclose(a, b, rel_tol=1e-09, abs_tol=0.0):
  return abs(a-b) <= max(rel_tol * max(abs(a), abs(b)), abs_tol)

# Test the orientation values
print("Old Orientation: " + str(originalOri))
print("New Orientation: " + str(newOri))
if (originalOri[0] != 0.0 or originalOri[1] != 0.0 or originalOri[2] != 0.0):
	raise Exception("Unexpected original orientation.")
if (not isclose(newOri[0], ROTATION_ANGLE) or newOri[1] != 0.0 or newOri[2] != 0.0):
	raise Exception("Unexpected new orientation.")


# Now test the body offsets using the beam
f = piData.getObjectOffset(torpBeamId);
if not f.isOk(1000):
	raise Exception("Failed getObjectOffset 1: " + f.responseDescription())
originalPos = f.value()

# Move the beam
f = piData.setObjectOffset(torpBeamId, [10.0, 200.0, 30.0])
if not f.isOk(1000):
	raise Exception("Failed setObjectOffset: " + f.responseDescription())

# Get the new position offset
f = piData.getObjectOffset(torpBeamId);
if not f.isOk(1000):
	raise Exception("Failed getObjectOffset 2: " + f.responseDescription())
newPos = f.value()

# Print values and do comparisons
print("Old Position: " + str(originalPos))
print("New Position: " + str(newPos))
if (originalPos[0] != 0.0 or originalPos[1] != 0.0 or originalPos[2] != 0.0):
	raise Exception("Unexpected original orientation.")
if (not isclose(newPos[0], 10.0) or not isclose(newPos[1], 200.0) or not isclose(newPos[2], 30.0)):
	raise Exception("Unexpected new position.")


# All done, successful if we got here
client.close()
print("Test successful")
