import RemoteApi_pb2
import remoteapi

"""
TestBookmark.py

Unit test for Remote API Bookmark functionality
"""

def initScenario(pi):
    piData = pi.data()
    f = piData.isScenarioInitialized()
    if not f.isOk(1000):
        raise Exception("Failed isScenarioInitialized")

    if f.value():
        f = piData.deleteScenario()
        if not f.isOk(5000):
            raise Exception("Failed to delete scenario")

    f = piData.initializeScenario("Bookmark Test", 0, 0, 0)
    if not f.isOk(5000):
        raise Exception("Failed to initialize scenario")

def getAndVerifyNumBookmarks(piSimdis, num):
    f = piSimdis.getAllBookmarkEvents()
    if not f.isOk(1000):
        raise Exception("getAllBookmarkEvents failed")

    if f.value().__len__() != num:
        raise Exception("Failed to get expected number of bookmarks")

    return f.value()

client = remoteapi.DiscoveryClient().createClient()
try:
    if client is None:
        raise Exception("Failed to connect.")

    pi = client.pluginApi()
    piSimdis = pi.simdis()

    initScenario(pi)

    pi.config().setResponsesEnabled(True)

    ############################################

    # Add some bookmarks
    f = piSimdis.addBookmarkEvent(0, "Comment 0", "Comment 0 Text")
    if not f.isOk(1000):
        raise Exception("addBookmarkEvent failed")

    f = piSimdis.addBookmarkEvent(0, "Comment 1", "Comment 1 Text", 89)
    if not f.isOk(1000):
        raise Exception("addBookmarkEvent failed")

    f = piSimdis.addBookmarkEvent(0, "Comment 2", "Comment 2 Text", 42)
    if not f.isOk(1000):
        raise Exception("addBookmarkEvent failed")

    # Attempt to add a bookmark with an invalid id (0)
    # Expected to fail, so raise an exception if it succeeds
    f = piSimdis.addBookmarkEvent(0, "Invalid", "Invalid Text", 0)
    if f.isOk(1000):
        raise Exception("addBookmarkEvent succeeded when it should have failed")

    # Verify number of bookmarks is correct
    getAndVerifyNumBookmarks(piSimdis, 3)

    # Remove a bookmark
    f = piSimdis.removeBookmarkEvent(42)
    if not f.isOk(1000):
        raise Exception("removeBookmarkEvent failed")

    # Verify number of bookmarks is correct
    getAndVerifyNumBookmarks(piSimdis, 2)

    # Verify that trying to get the bookmark we removed fails
    f = piSimdis.getBookmarkEventData(42)
    if f.isOk(1000):
        raise Exception("getBookmarkEventData succeeded when it should have failed")

    # Add a new bookmark with an id we already used but deleted
    f = piSimdis.addBookmarkEvent(0, "Comment 3", "Comment 3 Text", 42)
    if not f.isOk(1000):
        raise Exception("addBookmarkEvent failed")

    # Verify number of bookmarks is correct
    idList = getAndVerifyNumBookmarks(piSimdis, 3)

    # Remove two bookmarks using the returned vector from getAllBookmarkEvents()
    f = piSimdis.removeBookmarkEvent(idList[0])
    if not f.isOk(1000):
        raise Exception("removeBookmarkEvent failed")

    f = piSimdis.removeBookmarkEvent(idList[2])
    if not f.isOk(1000):
        raise Exception("removeBookmarkEvent failed")

    # Verify number of bookmarks is correct
    idList = getAndVerifyNumBookmarks(piSimdis, 1)

    # Verify the remaining bookmark event is 89
    if idList[0] != 89:
        raise Exception("Ids did not match as expected")

    # Add more bookmarks
    f = piSimdis.addBookmarkEvent(0, "Comment 4", "Comment 4 Text", 672)
    if not f.isOk(1000):
        print(f.responseDescription())
        raise Exception("addBookmarkEvent failed")

    f = piSimdis.addBookmarkEvent(0, "Comment 5", "Comment 5 Text")
    if not f.isOk(1000):
        raise Exception("addBookmarkEvent failed")

    f = piSimdis.addBookmarkEvent(0, "Comment 6", "Comment 6 Text", 24)
    if not f.isOk(1000):
        raise Exception("addBookmarkEvent failed")

    # Test retrieving a bookmark's data
    f = piSimdis.getBookmarkEventData(24)
    if not f.isOk(1000):
        raise Exception("getBookmarkEventData failed")

    data = f.value()
    if data.time != 0.0:
        raise Exception("Bookmark data time did not match")
    if data.category != "Comment 6":
        raise Exception("Bookmark data category did not match")
    if data.text != "Comment 6 Text":
        raise Exception("Bookmark data text did not match")

    # Test "updating" an existing bookmark by removing it and
    # then adding back with just the comment text changed
    f = piSimdis.removeBookmarkEvent(24)
    if not f.isOk(1000):
        raise Exception("removeBookmarkEvent failed")

    # Verify number of bookmarks is correct
    getAndVerifyNumBookmarks(piSimdis, 3)

    # Add the bookmark again with the new comment text
    newText = "New Comment 6 Text"
    f = piSimdis.addBookmarkEvent(data.time, data.category, newText, 24)
    if not f.isOk(1000):
        raise Exception("addBookmarkEvent failed")

    # Verify number of bookmarks is correct
    getAndVerifyNumBookmarks(piSimdis, 4)

    # Get the bookmark's data, and check it for correctness
    f = piSimdis.getBookmarkEventData(24)
    if not f.isOk(1000):
        raise Exception("getBookmarkEventData failed")

    data = f.value()
    if data.time != 0.0:
        raise Exception("Bookmark data time did not match")
    if data.category != "Comment 6":
        raise Exception("Bookmark data category did not match")
    if data.text != newText:
        raise Exception("Bookmark data text did not match")

    # Remove all bookmarks
    f = piSimdis.removeBookmarkEvent(0)
    if not f.isOk(1000):
        raise Exception("removeBookmarkEvent failed")

    # Verify all bookmarks have been deleted
    getAndVerifyNumBookmarks(piSimdis, 0)

    f = piSimdis.addBookmarkEvent(0, "Comment 6", "Comment 6 Text", 24)
    if not f.isOk(1000):
        raise Exception("addBookmarkEvent failed")

	# Test retrieving a bookmark in the TOC format
    f = piSimdis.getBookmarkEvent(24)
    if not f.isOk(1000):
        raise Exception("getBookmarkEventData failed")

    data = f.value()
    if data.time != 0.0:
        raise Exception("Bookmark data time did not match")
    if data.category != "Comment 6":
        raise Exception("Bookmark data category did not match")
    if data.text != "Comment 6 Text":
        raise Exception("Bookmark data text did not match")

    f = piSimdis.addBookmarkEvent(10, "__SIMDIS::SCREEN_TEXT_CHANGE", "\"Bookmark Demo\" \"impact.ttf\" 28 0xff00ffff 50 8 0 0 C T", 25)
    if not f.isOk(1000):
        raise Exception("addBookmarkEvent failed")

    # Test retrieving a bookmark in the TOC format
    f = piSimdis.getBookmarkEvent(25)
    if not f.isOk(1000):
        raise Exception("getBookmarkEventData failed")

    data = f.value()
    if data.time != 10.0:
        raise Exception("Bookmark data time did not match")
    if data.category != "__SIMDIS::SCREEN_TEXT_CHANGE":
        raise Exception("Bookmark data category did not match")
    if data.text != "\"Bookmark Demo\" \"impact.ttf\" 28 0xff00ffff 50 8 0 0 C T":
        raise Exception("Bookmark data text did not match")

    # Remove all bookmarks
    f = piSimdis.removeBookmarkEvent(0)
    if not f.isOk(1000):
        raise Exception("removeBookmarkEvent failed")

    # All done, successful if we got here
    print("Test successful")

finally:
    client.close()
