import math
import RemoteApi_pb2
import remoteapi
import PISIMDIS_pb2

"""
TestRCS.py

Unit tests for Remote API command relating to RCS
"""

client = remoteapi.DiscoveryClient().createClient()
if client is None:
	raise Exception("Failed to connect.")

pi = client.pluginApi()
piData = pi.data()
piSimdis = pi.simdis()

f = piData.deleteScenario()
if not f.isOk(1000):
	raise Exception("Failed deleteScenario")

# Load data file is unique because it can take a while to respond...
f = piData.loadDataFile("$(SIMDIS_DIR)/demos/RFProp/AREPS/RCSnAREPS.zip")
print("Requested load of data file, waiting up to 80 seconds")
if not f.isOk(80000):
	raise Exception("Failed loadDataFile: " + f.responseDescription())

# Find platform named Ship -- it will have RCS fake_rcs_3.rcs
f = piData.findPlatform('Ship')
if not f.isOk(1000) or f.value() == 0:
	raise Exception("Failed findPlatform on Ship")
shipId = f.value()

# Get the RCS value
f = piSimdis.getRCSdB(shipId, 15000.0, math.radians(130.0), 0.0, PISIMDIS_pb2.POLARITY_UNKNOWN)
if not f.isOk(1000):
	raise Exception("Failed getRCSdb")
rcsValue = f.value()

# Test the RCS value
print("Got RCS value: " + str(rcsValue) + " dB; expected 37.0 dB.")
if rcsValue != 37.0:
	raise Exception("Got unexpected RCS value " + str(rcsValue))

# Set the RCS data
f = piSimdis.setRCSData(shipId, PISIMDIS_pb2.POLARITY_LINEAR, 40)
if not f.isOk(10000):
    raise Exception("Failed setRCSData")

# Get the RCS data
f = piSimdis.getRCSData(shipId)
if not f.isOk(10000):
    raise Exception("Failed getRCSData")
rcsData = f.value()

# Test the polarity value
polarity = rcsData.polarity
print("Got RCS polarization code: " + str(polarity) + "; expected " + str(PISIMDIS_pb2.POLARITY_LINEAR))
if polarity != PISIMDIS_pb2.POLARITY_LINEAR:
    raise Exception("Got unexpected polarization code " + str(polarity))

# Test the RCS frequency value
freq = rcsData.freqMhz
print("Got RCS frequency: " + str(freq) + " Mhz; expected 40.0 Mhz")
if freq != 40:
    raise Exception("Got unexpected frequency " + str(freq))

# All done, successful if we got here
client.close()
print("Test successful")
