import math
import Constants_pb2
import PISIMDIS_pb2
import PISIMDISRangeCalculations_pb2
import RemoteApi_pb2
import PIDataReferenceFrame_pb2
import remoteapi
import ScenarioHelper

"""
TestRangeCalculateValue.py

Unit test for Remote API that creates a simple scenario and calls some range calculations.
"""
def areEqual(a, b, rel_tol=1e-06, abs_tol=0.0):
  return abs(a - b) <= max(rel_tol * max(abs(a), abs(b)), abs_tol)


def testValue(type, name, value):
	f = piSimdis.calculateEntityValue(type, 10, -1)
	if not f.isOk(1000):
		raise Exception(name + ' failed: ' + f.responseDescription())

	if not areEqual(f.value()[0], value):
		raise Exception(name + ' is not correct')


def testValues(type, name, value0, value1):
	f = piSimdis.calculateEntityValue(type, 10, -1)
	if not f.isOk(1000):
		raise Exception(name + ' failed: ' + f.responseDescription())

	if not areEqual(f.value()[0], value0):
		raise Exception(name + ' value0 is not correct')

	if not areEqual(f.value()[1], value1):
		raise Exception(name + ' value1 is not correct')


client = remoteapi.DiscoveryClient().createClient()

if client is None:
	raise Exception("Failed to connect.")

pi = client.pluginApi()
pi.config().setResponsesEnabled(True)
piData = pi.data()
piSimdis = pi.simdis()

ScenarioHelper.newScenario(piData, origin=[math.radians(22), math.radians(-159),0])
ScenarioHelper.createPlatformVelocity(client, 10, [math.radians(22), math.radians(-159), 0.0], [100.0, 200.0, 300.0])
ScenarioHelper.createPlatform(client, 20, [math.radians(22.1), math.radians(-159), 0.0]) # slightly north

# Fix time and wait for the response
piData.clockAutoConfigureBeginEnd().waitForResponse(1000)

# Now calculate some values
piCommon = pi.common()
f = piCommon.rangeCalculateValue(20, 10, Constants_pb2.RANGECALC_AZIMUTHTRUE)
if not f.isOk(1000):
	raise Exception(f.responseDescription())

degValue = math.degrees(f.value())

if degValue < 179.999 or degValue > 180.001:
	raise Exception('Failed azimuth calculation test.')

# Value as of 03/26/17
testValues(PISIMDISRangeCalculations_pb2.CALC_SUN_POSITION, 'Sun Position', 3.5751959361739125, 0.71297912198904689)
testValue(PISIMDISRangeCalculations_pb2.CALC_SUN_ILLUMINANCE, 'Sun Illuminance', 68272.580770335597)
testValues(PISIMDISRangeCalculations_pb2.CALC_MOON_POSITION, 'Moon Position', 1.2060636290146947, -0.19093813357940337)
testValue(PISIMDISRangeCalculations_pb2.CALC_MOON_ILLUMINANCE, 'Moon Illuminance', 0.0000000121911399034571683)
testValue(PISIMDISRangeCalculations_pb2.CALC_MACH, 'Mach', 1.0951193)
testValue(PISIMDISRangeCalculations_pb2.CALC_AOA, 'AoA', -0.93027403)
testValue(PISIMDISRangeCalculations_pb2.CALC_TOTAL_AOA, 'Total AoA', 1.00685371)
testValue(PISIMDISRangeCalculations_pb2.CALC_SIDE_SLIP, 'Side Slip', -0.46364763)
testValue(PISIMDISRangeCalculations_pb2.CALC_FPA, 'FPA', 0.93027399)
testValue(PISIMDISRangeCalculations_pb2.CALC_COURSE, 'Course', 0.46364760)

client.close()
print("Test successful")
