import math
import RemoteApi_pb2
import remoteapi

"""
TestSetSettings.py

Unit test for Remote API setSettings and getSettings functions
"""

client = remoteapi.DiscoveryClient().createClient()
if client is None:
    raise Exception("Failed to connect.")

piCommon = client.pluginApi().common()

# PICommon::DoesSettingExist tests ----------------------------
f = piCommon.doesSettingExist("HUD/Logo/Right Logo Filename")
if not f.isOk(1000):
    raise Exception("Failed to check HUD/Logo/Right Logo Filename")
if not f.value():
    raise Exception("HUD/Logo/Right Logo Filename returning non-existent")

f = piCommon.doesSettingExist("Console/MadeUp Setting")
if not f.isOk(1000):
    raise Exception("Failed to check MadeUp Setting")
if f.value():
    raise Exception("MadeUp Setting returning existent")

# PICommon::*SettingValueStr tests ----------------------------
setLogoPath = "green-flag.png"
f = piCommon.setSettingValueStr("HUD/Logo/Right Logo Filename", setLogoPath)
if not f.isOk(1000):
    raise Exception("Failed to set HUD/Logo/Right Logo Filename")

f = piCommon.getSettingValueStr("HUD/Logo/Right Logo Filename", "default")
if not f.isOk(1000):
    raise Exception("Failed to get HUD/Logo/Right Logo Filename")
if f.value() != setLogoPath:
    raise Exception("Fetched HUD/Logo/Right Logo Filename")

# PICommon::*SettingValueInt tests ----------------------------
setBufferLines = 1001
f = piCommon.setSettingValueInt("Console/Buffer Lines", setBufferLines)
if not f.isOk(1000):
    raise Exception("Failed to set Console/Buffer Lines")

f = piCommon.getSettingValueInt("Console/Buffer Lines", 0)
if not f.isOk(1000):
    raise Exception("Failed to get Console/Buffer Lines")
if f.value() != setBufferLines:
    raise Exception("Fetched Console/Buffer Lines does not match")

# PICommon::*SettingValueDouble tests ----------------------------
setLineWidth = 2.22
f = piCommon.setSettingValueDouble("GOG/OGR Line Width", setLineWidth)
if not f.isOk(1000):
    raise Exception("Failed to set GOG/OGR Line Width")

f = piCommon.getSettingValueDouble("GOG/OGR Line Width", 0.0)
if not f.isOk(1000):
    raise Exception("Failed to get GOG/OGR Line Width")
if f.value() != setLineWidth:
    raise Exception("Fetched GOG/OGR Line Width does not match")

# PICommon::*SettingValueBool tests ----------------------------
setOnTop = True
f = piCommon.setSettingValueBool("Console/Newest On Top", setOnTop)
if not f.isOk(1000):
    raise Exception("Failed to set Console/Newest On Top")

f = piCommon.getSettingValueBool("Console/Newest On Top", False)
if not f.isOk(1000):
    raise Exception("Failed to get Console/Newest On Top")
if f.value() != setOnTop:
    raise Exception("Fetched Console/Newest On Top does not match")

# PICommon::*SettingValueColor tests ----------------------------
setGOGColor1 = 0xffaaaaff
f = piCommon.setSettingValueColor("GOG/Color 1", setGOGColor1)
if not f.isOk(1000):
    raise Exception("Failed to set GOG/Color 1")

f = piCommon.getSettingValueColor("GOG/Color 1", 0x00000000)
if not f.isOk(1000):
    raise Exception("Failed to get GOG/Color 1")
if f.value() != setGOGColor1:
    raise Exception("Fetched GOG/Color 1 does not match")

# PICommon::*SettingValueStringVec tests ----------------------------
strList = ['123', '5', '321']
f = piCommon.setSettingValueStringVec("Private/StringList", strList)
if not f.isOk(1000):
    raise Exception("Failed to set Private/StringList")

f = piCommon.getSettingValueStringVec("Private/StringList")
if not f.isOk(1000):
    raise Exception("Failed to get Private/StringList")
if f.value() != strList:
    raise Exception("Fetched Private/StringList does not match")

# Test default value against a setting that should not exist
f = piCommon.doesSettingExist("Private/ShouldNotExist")
if not f.value():
    f = piCommon.getSettingValueStringVec("Private/ShouldNotExist", ['test'])
    if not f.isOk(1000):
        raise Exception("Failed to get Private/ShouldNotExist")
    if f.value() != ['test']:
        raise Exception("Fetched Private/ShouldNotExist does not match input default values")

# All done, successful if we got here
client.close()
print("Test successful")
