import RemoteApi_pb2
import PICommon_pb2
import remoteapi
import remoteapi.PluginApi

"""

TestSlotData.py

Unit test for Remote API that works with slot data; expects a file created by Simple Server
NOTE: Start Simple Server in File Mode before executing this test
"""
def ValidateVector(name, inData, outData):
	if len(inData) != len(outData):
		raise Exception(name + " lengths do not match: expected = " + str(len(inData)) + " got = " + str(len(outData)))

	for x in range(0, len(inData)):
		if inData[x] != inData[x]:
			raise Exception(name + "values at " + str(x) + " do not match: expected = " + str(inData[x]) + " got = " + str(outData[x]))


ReturnCodeOnly = lambda r: r.response.returnCode

client = remoteapi.DiscoveryClient().createClient()

if client is None:
	raise Exception("Failed to connect.")

pi = client.pluginApi()
pi.config().setResponsesEnabled(True)
piData = pi.data()

# Verify that a scenario is loaded
f = piData.isScenarioInitialized()
if not f.isOk(1000):
	raise Exception("isScenarioInitialized failed: " + f.responseDescription())

if not f.value():
	raise Exception("A Simple Server scenario needs to be loaded")

# Find a beam to add a RF stuff to
f = piData.findBeam("11 p-3c_orion_nrl beam");
if not f.isOk(1000):
	raise Exception("findBeam: " + f.responseDescription())
host = f.value();
if host == 0:
	raise Exception("A Simple Server scenario needs to be loaded")

# Get the start time
f = piData.clockGetBegin();
if not f.isOk(1000):
	raise Exception("clockGetBegin: " + f.responseDescription())
startTime = f.value();

# Need to prime the beam with some Radar parameters; the values are not realistic, but instead picked for ease of verification
msg = RemoteApi_pb2.Command()
radar = msg.piSimdis.rfprop.setRadarParams;
radar.beamId = host
radar.vbw = 1.0
radar.hbw = 1.1
radar.antGain = 2
radar.xmtPower = 3
radar.freq = 4
radar.prf = 5
radar.pulseWidth = 6
radar.systemLost = 7
radar.noiseFigure = 8
radar.antPattern = 1
radar.antPolarization = 2

# Send Radar data
f = remoteapi.PluginApi.ResponseHandler(client, client.send(msg), ReturnCodeOnly)
if not f.isOk(1000):
	raise Exception("setRadarParams: " + f.responseDescription())

# Create slot data; the values are not realistic, but instead picked for ease of verification
msg = RemoteApi_pb2.Command()
msg.piSimdis.rfprop.setSlotData.beamId = host
inData = msg.piSimdis.rfprop.setSlotData.data.add();
inData.time = startTime
inData.inUse = True
inData.fWaterSlot = False
inData.nrout = 3
inData.nzout = 4
inData.minRange = 10
inData.maxRange = 20
inData.minHeight = 30
inData.maxHeight = 40
inData.antHeight = 5
inData.az = 1
inData.el = 0.5
gh = inData.gndHgtMap.add()
gh.groundRange = 10
gh.heightHae = 30
gh = inData.gndHgtMap.add()
gh.groundRange = 15
gh.heightHae = 35
gh = inData.gndHgtMap.add()
gh.groundRange = 20
gh.heightHae = 40
inData.grazeAng.append(1)
inData.grazeAng.append(2)
inData.grazeAng.append(3)
inData.cnrDb.append(11)
inData.cnrDb.append(12)
inData.cnrDb.append(13)
inData.loss1m.append(21)
inData.loss1m.append(22)
inData.loss1m.append(23)
inData.ppf1m.append(31)
inData.ppf1m.append(32)
inData.ppf1m.append(33)
inData.ppf.append(411)
inData.ppf.append(412)
inData.ppf.append(413)
inData.ppf.append(421)
inData.ppf.append(422)
inData.ppf.append(423)
inData.ppf.append(431)
inData.ppf.append(432)
inData.ppf.append(433)
inData.ppf.append(441)
inData.ppf.append(442)
inData.ppf.append(443)
inData.ploss.append(511)
inData.ploss.append(512)
inData.ploss.append(513)
inData.ploss.append(521)
inData.ploss.append(522)
inData.ploss.append(523)
inData.ploss.append(531)
inData.ploss.append(532)
inData.ploss.append(533)
inData.ploss.append(541)
inData.ploss.append(542)
inData.ploss.append(543)
inData.ppfEl.append(61)
inData.ppfEl.append(62)
inData.ppfEl.append(63)

# Send Slot data
f = remoteapi.PluginApi.ResponseHandler(client, client.send(msg), ReturnCodeOnly)
if not f.isOk(1000):
	raise Exception("setSlotData: " + f.responseDescription())

if f.value() != 0:
	raise Exception("setSlotData returned: " + str(f.value()))

# Now get the slot data
getMsg = RemoteApi_pb2.Command()
getMsg.piSimdis.rfprop.getSlotData.beamId  = host
getMsg.piSimdis.rfprop.getSlotData.trueAzimuth  = 1

f = remoteapi.PluginApi.ResponseHandler(client, client.send(getMsg),
	lambda r: r.piSimdis.rfprop.getSlotData)
if not f.isOk(1000):
	raise Exception("getSlotData: " + f.responseDescription())
outData = f.value()[0]

# The get did not actually get everything so comment out the things not supported so it is documenting what was and what was not retrieved

# time is always set to -1
if outData.time != -1:
	raise Exception("time: expected = -1 got = " + str(outData.time))

#if inData.inUse != outData.inUse:
#	raise Exception("inUse: expected = " + str(inData.inUse) + " got = " + str(outData.inUse))

#if inData.fWaterSlot != outData.fWaterSlot:
#	raise Exception("fWaterSlot: expected = " + str(inData.fWaterSlot) + " got = " + str(outData.fWaterSlot))

if inData.nrout != outData.nrout:
	raise Exception("nrout: expected = " + str(inData.nrout) + " got = " + str(outData.nrout))

if inData.nzout != outData.nzout:
	raise Exception("nzout: expected = " + str(inData.nzout) + " got = " + str(outData.nzout))

if inData.minRange != outData.minRange:
	raise Exception("minRange: expected = " + str(inData.minRange) + " got = " + str(outData.minRange))

if inData.maxRange != outData.maxRange:
	raise Exception("maxRange: expected = " + str(inData.maxRange) + " got = " + str(outData.maxRange))

if inData.minHeight != outData.minHeight:
	raise Exception("minHeight: expected = " + str(inData.minHeight) + " got = " + str(outData.minHeight))

if inData.maxHeight != outData.maxHeight:
	raise Exception("maxHeight: expected = " + str(inData.maxHeight) + " got = " + str(outData.maxHeight))

if inData.antHeight != outData.antHeight:
	raise Exception("antHeight: expected = " + str(inData.antHeight) + " got = " + str(outData.antHeight))

if inData.az != outData.az:
	raise Exception("az: expected = " + str(inData.az) + " got = " + str(outData.az))

#if inData.el != outData.el:
#	raise Exception("el: expected = " + str(inData.el) + " got = " + str(outData.el))

#ValidateVector("gndHgtMap", inData.gndHgtMap, outData.gndHgtMap)
#ValidateVector("grazeAng", inData.grazeAng, outData.grazeAng)
ValidateVector("cnrDb", inData.cnrDb, outData.cnrDb)
#ValidateVector("loss1m", inData.loss1m, outData.loss1m)
#ValidateVector("ppf1m", inData.ppf1m, outData.ppf1m)
ValidateVector("ppf", inData.ppf, outData.ppf)
ValidateVector("ploss", inData.ploss, outData.ploss)
#ValidateVector("ppfEl", inData.ppfEl, outData.ppfEl)

client.close()
print("Test successful")
