import math
import RemoteApi_pb2
import remoteapi
import remoteapi.PluginApi
import ScenarioHelper
import time

"""
TestView.py

Unit test for Remote API View command functionality
"""

JP_longitude = 135.744
JP_latitude = 38.9309
viewString = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<Simdis10ViewFile>
<Version>1.0</Version>
<EyePosition>
<Name>EyePos_Japan</Name>
<Longitude>{}</Longitude>
<Latitude>{}</Latitude>
<Altitude>-2170.68</Altitude>
<Azimuth>-5.11181</Azimuth>
<Elevation>-81.6109</Elevation>
<Range>1.58575e+006</Range>
<Overhead>False</Overhead>
<Cockpit>False</Cockpit>
<ReplaceCentered>False</ReplaceCentered>
<TetherMode>Center</TetherMode>
<LockHeading>False</LockHeading>
<LockPitch>False</LockPitch>
</EyePosition>
</Simdis10ViewFile>
""".format(JP_longitude, JP_latitude)

client = remoteapi.DiscoveryClient().createClient()
try:
	if client is None:
		raise Exception("Failed to connect.")

	pi = client.pluginApi()
	piData = pi.data()
	piSimdis = pi.simdis()

	ScenarioHelper.newScenario(piData, origin=[math.radians(22), math.radians(-159),0])
	f = ScenarioHelper.createPlatform(client, 20, [math.radians(22.1), math.radians(-159), 0.0]) # slightly north
	if not f.isOk(2500):
		raise Exception("Failed createPlatform: " + f.responseDescription())
	platId = f.value().piData.platform.uniqueId

	# Load view file with multiple viewports and eyePositions
	print("Requesting load of view file.")
	f = piSimdis.loadViewFile("$(SIMDIS_DIR)/demos/Plugins/VPHotkey/sample.view")
	if not f.isOk(45000):
		raise Exception("File load failed")

	f = piSimdis.cycleEyeView(True, "Corner2")
	if not f.isOk(3000):
		raise Exception("Failed to cycleEyeView: " + f.responseDescription())

	f = piSimdis.createView(viewString)
	if not f.isOk(1500):
		raise Exception("Failed to createView: " + f.responseDescription())

	f = piSimdis.changeView("EyePos_Japan", "MainView")
	if not f.isOk(1500):
		raise Exception("Failed to changeView: " + f.responseDescription())

	f = piSimdis.getCurrentViewport()
	if not f.isOk(1500):
		raise Exception("Failed to getCurrentViewport: " + f.responseDescription())
	if f.value() != "MainView":
		raise Exception("CurrentViewport does not match")

	f = piSimdis.setViewportFov("MainView", 40)
	if not f.isOk(1500):
		raise Exception("Failed to setViewportFov: " + f.responseDescription())

	f = piSimdis.getViewString()
	if not f.isOk(1500):
		raise Exception("Failed to getViewString: " + f.responseDescription())
	a = f.value().split("Longitude>")[1]
	lonStr = a[:a.find("<")]
	b = f.value().split("Latitude>")[1]
	latStr = b[:b.find("<")]
	if float(lonStr) != JP_longitude or float(latStr) != JP_latitude:
		raise Exception(("Received lat/lon {}/{}do not match {}/{}").format(latStr, lonStr, JP_latitude, JP_longitude))

	msg = RemoteApi_pb2.Command()
	camInfoMsg = msg.piSimdis.view.setCameraInfo
	camInfoMsg.viewport = "Corner1"
	camInfoMsg.cameraCenterPos.latInRadians = 13
	camInfoMsg.cameraCenterPos.lonInRadians = 175
	camInfoMsg.cameraCenterPos.altMeters = 100
	camInfoMsg.cameraRAE.rangeMeters = 5300000
	camInfoMsg.cameraRAE.azimuthRad = 5.25
	camInfoMsg.cameraRAE.elevationRad = 1.41
	f = remoteapi.PluginApi.ResponseHandler(client, client.send(msg), lambda r: r.response.returnCode)
	if not f.isOk(1500):
		raise Exception("Failed to setCameraInfo(1): " + f.responseDescription())

	f = piSimdis.setViewportFov("MainView", 40, 120)
	if not f.isOk(1500):
		raise Exception("Failed to setViewportFov: " + f.responseDescription())


	msg = RemoteApi_pb2.Command()
	camInfoMsg = msg.piSimdis.view.setCameraInfo
	camInfoMsg.viewport = "MainView"
	camInfoMsg.tetherId = platId
	f = remoteapi.PluginApi.ResponseHandler(client, client.send(msg), lambda r: r.response.returnCode)
	if not f.isOk(1500):
		raise Exception("Failed to setCameraInfo(2): " + f.responseDescription())

	f = piSimdis.getCameraInfo()
	if not f.isOk(1500):
		raise Exception("Failed to getCameraInfo: " + f.responseDescription())
	if f.value().viewport != "MainView":
		raise Exception("GetCameraInfo returned incorrect viewport")
	if f.value().tetherId != platId:
		raise Exception(("GetCameraInfo received tetherId: {}, expecting {}").format(f.value().tetherId, platId))

	f = piSimdis.getEyePosition()
	if not f.isOk(1500):
		raise Exception("Failed to getEyePosition: " + f.responseDescription())
	if abs(f.value().altMeters - 1572157.56435) > 0.1:
		raise Exception("GetEyePosition returned incorrect altMeters")

	f = piSimdis.viewMotion("CaliView", "Corner3", duration = 3.0)
	if not f.isOk(1500):
		raise Exception("Failed viewMotion: " + f.responseDescription())

	print("Sleeping before motionStop")
	time.sleep(0.5)

	f = piSimdis.motionStop("Corner3")
	if not f.isOk(1500):
		raise Exception("Failed motionStop: " + f.responseDescription())

	f = piSimdis.getOverheadMode("MainView")
	if not f.isOk(1500):
		raise Exception("Failed getOverheadMode: " + f.responseDescription())

	print("Test Successful")
finally:
	client.close()
